'*************************************************************
'                READ BELOW - IMPORTANT!!!
'*************************************************************
'***** This script REQUIRES WINZIP to be installed on   ******
'***** the local machine.  It also requires the command ******
'***** line plugin for winzip.	The winzip command line ******
'***** plugin *MUST* be installed.  You can verify that ******
'***** the plugin is installed by checking for the      ******
'***** existence of the following file:                 ******
'*****    C:\Program Files\WinZip\WZCLINE.GID           ******
'*****                                                  ******
'***** script will loop through subdirs in target dir   ******
'***** and zips up all files older than specified.      ******
'***** script looks within each subdir and creates      ******
'***** a single zip archive inside that is called       ******
'***** 'subdir.zip' which will contain all the files    ******
'***** older than one year.                             ******
'*****                                                  ******
'***** The script doesn't zip files directly in the     ******
'***** target dir, only files inside subdirectories.    ******
'*****                                                  ******
'***** After script completes, it will write a log file ******
'***** to the target directory (be sure user running    ******
'***** the script has sufficient perms to create and    ******
'***** modifiy files in target dir.                     ******
'*****                                                  ******
'***** The Script can be configured to email someone    ******
'***** after it completes, and will also email script   ******
'***** failures.  But this fuctionaity is disabled by   ******
'***** default since some email servers will have       ******
'***** configuration issues.  If you want to use this   ******
'***** feature, set lTurnEmailOn = 1 but be sure to     ******
'***** enter the correct parameters for your email      ******
'***** address and server, etc.                         ******
'*****                                                  ******
'***** See comments below for running from cmd line.    ******
'*****                                                  ******
'***** Written: 11/05/2006                              ******
'***** Author: Robert "Dude" Spellings Jr.              ******
'***** http://www.spellingsconsulting.com               ******
'*************************************************************
'*************************************************************


' declare vars and objects
Dim cNetCommands, vFolderName, vDay, oFolderP, oSubfolderOfP, aSubfolderOfP, oLogFile
Dim vLogLocation, vDate, vMapDrive, vOldDate, objEmail, sMachine, vDir, vTempZip , vReturnCode
Dim nTotalFilesProcessed, nFolderCounter, vElapsedTime, nTotCounter, vWinZipDir, vFileExt
dim vArgList, nFirstComma, nSecondComma, nRestOfList, nGetRightChars, nArchiveInterval, lTurnEmailOn


' ************************************************
' ***** Running From Command Line Arguments ******
' ************************************************
' if you run this script from the command line, you have the option of defining the paramerters
' of the script through command line arguments.  the syntax for assigning the values through the cmd line is:
' archive_files [vDir] [nArchiveInterval] [vFileExtList]
' example: archive_files "c:\Documents and Settings\administrator\My Documents" 180 DOC TXT RTF XLS PPT HTML 
' Explanation: 
' 	vDir = string. the full path of the target directory where you want the subdirs to be archived.  if the path has spaces in it, it must be contained by quotes.
'	nArchiveInterval = number.  this is age of the files to be archived in days, defined by the Last Modified Date.  ie: all files with last Modified dates older than nArchiveInterval will be archived.
'	vFileExtList = string. this is a list of file extensions that should be included in the archive.  Include as many extensions as needed, separate multiple extensions by spaces.  Case Insensitive.
' if one or more arguments are not included when calling the script, the script will continue using the *Default Values* defined below.

' ************************************
' ****  Start Get CMD Line Args   ****
' ************************************
' get file extension list from arguments passed.
Set oArgs = WScript.Arguments
' if args present, then loop through them and separate them out.
if oArgs.count > 0 then
	' loop through arguments and assign to pipe separated list
	For i = 0 to oArgs.Count - 1
	vArgList = vArgList & "," & oArgs(i)
	Next
	
	' if oArgs.count = 1 then just assign vDir
	if oArgs.count >= 1 then
		' yank out preceding comma in list of file extensions
		vArgList = replace(vArgList, ",","",1,1)
		' now define vars based on args
		vDir = vArgList  ' define target dir
		nArchiveInterval = 365  ' define default archive interval
		vFileExt = ""  ' define default file extension list filter
	end if
	
	if oArgs.count >= 2 then
		' if there is a comma in the list (which there should be if more than one arg) then get the first value and assign to vDir, and get the second value and assign to nArchiveInterval
		if InStr(1, vArgList, ",") > 0 then 
			' find the first comma and assign vDir to chars to the left of that comma.
			nFirstComma = InStr(1, vArgList, ",") 
			' assign the first arg to the vDir var (whatever is to the left of the first comma)
			vDir = Left(vArgList, nFirstComma-1)
			' subtract nFirstComma from Total Length to find remaining string, which will be nArchiveInterval
			nGetRightChars = len(vArgList) - nFirstComma
			' assign whatever is remaining to the nArchiveInterval var
			nArchiveInterval = Right(vArgList, nGetRightChars)
			' now define default file extension list filter
			vFileExt = ""
		end if 
	end if
	if oArgs.count >= 3 then
		' check for the second comma (which should be there if more than two
		if InStr(nFirstComma+1, vArgList, ",") > 0 Then 
			' find second comma, start searching from position of first comma
			nSecondComma = InStr(nFirstComma+1, vArgList, ",")
			' get everything to the left of the second comma
			nArchiveInterval = Left(vArgList, nSecondComma-1)
			' redefine first comma since string has changed
			nFirstComma = InStr(1, nArchiveInterval, ",")
			' then subtract nFirstComma from Total Length to find out how many chars are to the right of first comma, which should be second arg, or nArchiveInterval.
			nGetRightChars = len(nArchiveInterval) - nFirstComma
			' now take everything to the right of the first comma
			nArchiveInterval = Right(nArchiveInterval,nGetRightChars)
			' now define the rest of list, or everything to the right of the second comma, which should all be file extensions
			nRestOfList = Len(vArgList) - nSecondComma
			' now override default for file extension list filter
			vFileExt = Right(vArgList,nRestOfList)
			
		end if 
	end if
Else 
	' ********************************
	' *****    Default Values   ******
	' ********************************
	' if no args passed, then define defaults here
	' if you want to run the script as a scheduled task or by double-clicking, then change these values to suit your needs
	vDir = "c:\Documents and Settings\administrator\My Documents"

	' set file extension filter.  Examples: "jpg", "doc txt pdf txt", etc. Leave blank to include all files. Case Insensitive.
	vFileExt = ""
	
	' ** set var for archive interval in days (365 days = 1 year).
	' ** use this to change which files are add to the zip file.
	' ** files that have not been modified since this number of days will be zipped.
	nArchiveInterval = 365
end if

' ** set var for email recipient. this is only used if the email section below is uncommented and configured.
vEmailRecipient = "your_name@yourdomain.com"


' ** set more default values:

' set this to 1 if you want to turn on email capabilities (script will email results)
lTurnEmailOn = 0
' set date var
vDate = now()
' set var for nArchiveInterval days ago
vOldDate = vDate - nArchiveInterval
' set date var for use in filenames
vMonth = DatePart("m", vDate)
vDay = DatePart("d", vDate)
vYear = DatePart("yyyy", vDate)
' add preceding zero for filename consistency
if vDay < 10 then 
	vDay = "0" & vDay
end if 	
if vMonth < 10 then 
	vMonth = "0" & vMonth
end if
strDate = vMonth & vDay & vYear
' set var for winzip dir
vWinZipDir = "C:\Program Files\WinZip\"


' create objects
Set cNetCommands = CreateObject("WScript.Network")
Set cFSCommands = CreateObject("Scripting.FileSystemObject")
Set cShellCommands = CreateObject("WScript.Shell")
' get machine name
Set ObjEnv = cShellCommands.Environment("Process")
'set var for computername to include on log file and emails.
sMachineName = ObjEnv("COMPUTERNAME")

'create object for log file  
set oLogFile = cFSCommands.CreateTextFile(vDir&"\zip_log"&strDate&".txt", True)
' write date to log file
oLogFile.WriteLine("Log File for archive_files.vbs.  Script Start Time: " & vDate & ".") 
oLogFile.WriteLine("Running on Machine: " & sMachineName & ".")
oLogFile.WriteLine("Files Not Modified since " & vOldDate & " will be added to zip archive.") 

' check for winzip commandline file and send email if not exists
If cFSCommands.FileExists(vWinZipDir + "WZCLINE.GID") = true Then
	oLogFile.WriteLine("Winzip command line plugin present at " + vWinZipDir + "WZCLINE.GID")
else 
	' check to see if email is turned on
	if lTurnEmailOn = 1 then
		Set objMessage = CreateObject("CDO.Message") 
		objMessage.Subject = "Script Failure" 
		objMessage.From = "archive_files.vbs@yourdomain.com" 
		objMessage.To = vEmailRecipient 
		objMessage.TextBody = "The archive_files.vbs script running as a scheduled task on " & sMachineName & " failed to execute because the Winzip Command Line plugin is missing." & Chr(10) & Chr(10) & "This script tested for the existence of " + vWinZipDir + "WZCLINE.GID and was unable to find the file locally on " & sMachineName & "." & Chr(10) & Chr(10) & "Please install the Winzip Command Line plugin directly on " & sMachineName & " for this script to run properly.  You may download the Command Line plugin here: http://www.winzip.com/prodpagecl.htm.  It requires Winzip 10 Pro.  The Shareware version of Winzip will not work." & Chr(10) & Chr(10) & "The Log File for this script can be found at:" & vDir & "\zip_log"&strDate&".txt"
		'==This section provides the configuration information for the remote SMTP server.
		'==Normally you will only change the server name or IP.
		objMessage.Configuration.Fields.Item _
		("http://schemas.microsoft.com/cdo/configuration/sendusing") = 2 
		
		'Name or IP of Remote SMTP Server
		objMessage.Configuration.Fields.Item _
		("http://schemas.microsoft.com/cdo/configuration/smtpserver") = "smtpserver.yourdomain.com"
		
		'Server port (typically 25)
		objMessage.Configuration.Fields.Item _
		("http://schemas.microsoft.com/cdo/configuration/smtpserverport") = 25 
		
		objMessage.Configuration.Fields.Update
		
		'==End remote SMTP server configuration section==
		' send the message
		objMessage.Send
		
		' destroy objects and abort the script since winzip not present
		 Set cNetCommands = nothing
		 Set cFSCommands = nothing
		 Set cShellCommands = nothing
		 Set wshshell = nothing
		 Set objMessage = nothing
		 Set ObjEnv =nothing
	 end if 
	 
	 ' write failure to log file.
	 oLogFile.WriteLine("")
	 oLogFile.WriteLine("************************************************************")
	 oLogFile.WriteLine("* Script Failed.  Winzip Command Line Plugin Not Installed *")
	 oLogFile.WriteLine("************************************************************")
	 oLogFile.WriteLine("")
	 oLogFile.WriteLine("Script End Time: " & Now())
	 ' now destroy objecdt for log file.
	 set oLogFile = nothing 
	 ' now abort script
	 wscript.quit()
end if

' write target dir to log file
oLogFile.WriteLine("Target Directory: " & vDir )
' write File Extension Filter to logfile
if Len(vFileExt) >= 1 then 
	oLogFile.WriteLine("File Extensions to Include: " & vFileExt )
Else
	oLogFile.WriteLine("No File Extension Filter Defined.  All Files will be Processed")
end if
' add separator to separate log header from body and make log file more readable
oLogFile.WriteLine("***************************************************************************************************")


' set oFolderP object to vDir.
Set oFolderP = cFSCommands.GetFolder(vDir)

' loop through all subdirs of vDir and create "temp" folder, and robocopy all
' files with a last-modified date more than nArchiveInterval days old to the "temp" subdir.  These 
' files will be zipped then moved back to the subdir as one single zip file.

' set folder counter to zero
nFolderCounter = 0

' set total file countt o zero
nTotalFilesProcessed = 0

' start loop through subdirs of vDir - (loop 1)
For Each objSubFolder in oFolderP.SubFolders

		' increment folder counter
		nFolderCounter = nFolderCounter + 1
		' write blank line log file after header and between folders to make it easier to read
		oLogFile.WriteLine("")
		' write folder name to log file so that we can see what the script is doing
		oLogFile.WriteLine("Start Processing "& objSubFolder & ":")
	
		' set var for cursory look through subdirs to see if there are any files w/LastModified older than nArchiveInterval.
		nPreCounter = 0 
			
		' start nested loop through files in subdir of vDir to see if there are any files w/LastModified older than nArchiveInterval
		For Each objFileItem in objSubFolder.Files
			' check if files match the vFileExt filter.   
	  		if Len(vFileExt) < 1 OR InStr(UCase(vFileExt), UCase(cFSCommands.GetExtensionName(objFileItem))) > 0 then 
				' look at lastModified date of file.  if LastModified older than nArchiveInterval days, increment counter.
				If objFileItem.DateLastModified < vOldDate Then
					nPreCounter = nPreCounter +1
				end if
			end if
		Next
		
		' only try to process files in the folder if nPreCounter > 0 (if there are files w/LastModified older than nArchiveInterval days)
		If nPreCounter > 0 Then
			' check for temp dir in each subdir.  if not exists, create.  
			If cFSCommands.FolderExists(objSubFolder & "\temp") = true Then
				' write line to log file to indicate temp dir already there,
				oLogFile.WriteLine(chr(9) & "Temp dir already present.  Do Not Create temp dir: " & objSubFolder & "\temp")
			Else
				'create temp dir
				Set objFolder = cFSCommands.CreateFolder(objSubFolder & "\temp")
				
				'write to log file.
				oLogFile.WriteLine(chr(9) & "Temp dir not present.  Creating temp dir: " & objSubFolder & "\temp")
				
			End If 
			' end check for temp dir
				
			' set var for folder name to use in next if/then
			vZipName = objSubFolder.name
				
			' check for zip file.  if not exists write to log file, if exists move to temp folder and rename to "temp.zip"
			If cFSCommands.FileExists(objSubFolder & "\" & vZipName & ".zip" ) = true Then
				oLogFile.WriteLine(chr(9) & objSubFolder & "\" & vZipName & ".zip already present.")
				oLogFile.WriteLine(chr(9) & "Moving file to  " & objSubFolder & "\temp and rename to temp.zip")
				' Move zip file to temp folder and rename
			  	cFSCommands.MoveFile objSubFolder & "\" & vZipName & ".zip", objSubFolder & "\temp\temp" & ".zip"
			  	' write results to log file
			  	oLogFile.WriteLine(chr(9) & "Moved temp.zip file to  " & objSubFolder & "\temp successfully.")
				' set var to indicate zip file alrready existed
				vTempZip = 1
			else
				oLogFile.WriteLine(chr(9) & objSubFolder & "\" & vZipName & ".zip not present.  Temp.zip will be created in temp directory.")
				' set var to indicate no zip existed
				vTempZip = 0
			end if
			' end check for .zip file
				
			set objTemp = cFSCommands.GetFolder(objSubFolder & "\temp")
				
			' set counters to zero. (count files processed in order to add to log file)
			' nCounter counts fiels processed
			nCounter = 0 
			' nTotCounter counts all the files in the dir, so we can report how many files were skipped.
			nTotCounter = 0
			' start nested loop through files of the subdir of vDir - (loop 2)
			For Each objFileItem in objSubFolder.Files
				' increment nTotCounter
				nTotCounter = nTotCounter + 1
				' look at lastModified date of file.  if LastModified older than nArchiveInterval days, move to temp dir.
				If objFileItem.DateLastModified < vOldDate Then
					' verify that file has the right file extension or the file extension list is empty
					if Len(vFileExt) < 1 OR InStr(UCase(vFileExt), UCase(cFSCommands.GetExtensionName(objFileItem))) > 0 then   
						' assign last modified date
					 	vLastMod = objFileItem.DateLastModified
								  
						' increment counter only if last modified date is older than nArchiveInterval days
						nCounter = nCounter +1
								    
						'assign filename to var
						sFileName = cFSCommands.GetFileName(objFileItem)
								  
						sSourceFile = objFileItem
						sDestinationFile = objSubFolder & "\temp\" & sFileName
								  
						' Move old file to temp folder
						cFSCommands.MoveFile sSourceFile, sDestinationFile
								  
						'write to log file
						oLogFile.WriteLine(chr(9) & sFileName & " moved to temp subdirectory for archiving.  Last Modified Date: " & vLastmod)
					end if	  
				end if
				' end if for moving files w/LastModified older than nArchiveInterval days.
			
			Next
			' end nested loop - (loop 2)
						
			' zip all files in the temp subdir.  This must be done inside windows shell.
			DIM WshShell
		  	SET WshShell = CreateObject("WScript.Shell")	
			CreateObject("Wscript.Shell").CurrentDirectory = objTemp
			
			' -mu switch for winzip:  Move all files to temp.zip.  Those files that already existed 
			' in temp.zip and are newer on disk, update those files in zip.  Any new files found in the 
			' current folder are also moved to temp.zip.  Files on disk are deleted.
			' set var for windows shell return code, in order to make vbs wait for winzip via WHILE command.
			vReturnCode = WshShell.Run("wzzip -mu temp.zip *.*", 1, TRUE)
		
			' only wait for temp file to be created if no zip file existed before (if vTempZip = 0)
			If vTempZip = 0 then
			  	' wait here till temp.zip file is created
			  	do while cFSCommands.FileExists(objTemp & "\temp.zip") = false
			  		wscript.sleep 2000
			  	loop
				oLogFile.WriteLine(chr(9) & objSubFolder & "\temp\temp.zip created and " & nCounter & " files added to Zip")
			else
				' otherwise if vTempZip != 0 then wait based on number of files.  Wait 1/50 second for each file.
				' set var for length of time (no. files * 1/100 sec)
				nSleepTime = nCounter * 10
				' do the sleep command
				wscript.sleep nSleepTime
				
				'write results to log file
				oLogFile.WriteLine(chr(9) & nCounter & " added to existing zip file.")
			end if
			
			' write to log file
			oLogFile.WriteLine(chr(9) & nCounter & " Files Processed in " & objSubFolder)
			oLogFile.WriteLine(chr(9) & nTotCounter & " Total Files in " & objSubFolder)
			' calculate how many files skipped
			nSkipped = nTotCounter - nCounter
			' write to log
			oLogFile.WriteLine(chr(9) & nSkipped & " Files skipped in " & objSubFolder)
			' rename temp zip file to foldername.zip
		  	NewZipName = objSubFolder.name
		  	set cZipFile = cFSCommands.GetFile(objTemp & "\temp.zip")
		  	cZipFile.name = NewZipName & ".zip"
			
			' write results to log file
			oLogFile.WriteLine(chr(9) & objSubFolder & "\temp\temp.zip renamed to " & NewZipName & ".zip")
		  
		
			' move zip file back to parent dir
			' write to log file
			oLogFile.WriteLine(chr(9) & "Moving " & NewZipName & ".zip back to " & objSubFolder)
			
		   	'assign filename to var
		  	 sZipFileName = cFSCommands.GetFileName(cZipFile)
				  
		 	sSourceFile = objSubFolder & "\temp\" & sZipFileName
		 	sDestinationFile =  objSubFolder & "\" & sZipFileName
				  
		 	' do the move
		 	cFSCommands.MoveFile sSourceFile, sDestinationFile
		 
		 	' write move results to log file.
			oLogFile.WriteLine(chr(9) & "Moved " & NewZipName & ".zip back to " & objSubFolder & " successfully")
			
		 	' create another shell here to change directories so that permissions behave correctly, then destroy shell object
		 	CreateObject("Wscript.Shell").CurrentDirectory = "c:\"
		 	SET WshShell = nothing

			' delete the temp folder if exists
		 	if cFSCommands.FolderExists(objSubFolder & "\temp") then
			 	oLogFile.WriteLine(chr(9) & "Deleting Temp Dir " & objSubfolder.path & "\temp")
		 	 	cFSCommands.DeleteFolder objTemp
				 oLogFile.WriteLine(chr(9) & "Deleted Temp Dir in " & objSubFolder & " successfully")
			end if
			
			' destroy objtemp to make way for next dir
			set objTemp=nothing
			
			' add file count to Total
			nTotalFilesProcessed = nTotalFilesProcessed + nCounter
			
		' else if nPreCunter =0 write results to log file
		else
			oLogFile.WriteLine(chr(9) & "No Files eligible to archive in zip file.  All files have been modified since "& vOldDate)
			oLogFile.WriteLine(chr(9) & nPreCounter & " files processed in " &  objSubFolder)
		end if
		' end if for nPrecounter > 0
		
Next
' end loop through subdirs - (loop 1)

'write spacer line to log file
oLogFile.WriteLine("")

' write ending time to log file for performance monitoring
oLogFile.WriteLine("************************************************************************")
oLogFile.WriteLine(nTotalFilesProcessed & " files processed in " & nFolderCounter & " folders")
' calculate elepased time and set var
vElapsedTime = DateDiff("s",vDate,Now())
' write time infor to log file
oLogFile.WriteLine("Script End Time: " & Now())
oLogFile.WriteLine("Total Elapsed Time: " & vElapsedTime & " seconds")

' close log file and destroy object
oLogFile.close
set oLogFile=nothing

' check to see if email is turned on.  if so, email results.
if lTurnEmailOn = 1 then
	' send email with results of script
	Set objMessage = CreateObject("CDO.Message") 
	objMessage.Subject = "Script Successfully Completed" 
	objMessage.From = "archive_files.vbs@yourdomain.com" 
	objMessage.To = vEmailRecipient 
	objMessage.TextBody = "The archive_files.vbs script running as a scheduled task on " & sMachineName & " has successfully completed." & Chr(10) & Chr(10) & "There were " & nTotalFilesProcessed & " files processed in " & nFolderCounter & " folders." & Chr(10) & Chr(10) &  "The task started at " & vDate & " and took " & vElapsedTime & " seconds to complete." & Chr(10) & Chr(10) & "The Log File for this script can be found at:" & vDir & "\zip_log"&strDate&".txt" 
	'==This section provides the configuration information for the remote SMTP server.
	'==Normally you will only change the server name or IP.
	objMessage.Configuration.Fields.Item _
	("http://schemas.microsoft.com/cdo/configuration/sendusing") = 2 
	
	'Name or IP of Remote SMTP Server
	objMessage.Configuration.Fields.Item _
	("http://schemas.microsoft.com/cdo/configuration/smtpserver") = "smtpserver.yourdomain.com"
	
	'Server port (typically 25)
	objMessage.Configuration.Fields.Item _
	("http://schemas.microsoft.com/cdo/configuration/smtpserverport") = 25 
	
	objMessage.Configuration.Fields.Update
	
	'==End remote SMTP server configuration section==
	' send the message
	objMessage.Send
end if 
	
'destroy REMAINING objects
set cFSCommands=nothing	
set cNetCommands=nothing
set cShellCommands=nothing


'END SCRIPT